package org.fossasia.openevent.general.settings

import android.arch.lifecycle.ViewModel
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.CompositeDisposable
import io.reactivex.schedulers.Schedulers
import org.fossasia.openevent.general.auth.AuthService
import timber.log.Timber

class SettingsFragmentViewModel(private val authService: AuthService) : ViewModel() {

    private val compositeDisposable = CompositeDisposable()

    fun isLoggedIn() = authService.isLoggedIn()

    fun logout() {
        compositeDisposable.add(authService.logout()
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe({
                    Timber.d("Logged out!")
                }) {
                    Timber.e(it, "Failure Logging out!")
                })
    }

    fun getMarketAppLink(packageName: String): String{
        return "market://details?id=" + packageName
    }

    fun getMarketWebLink(packageName: String): String{
        return "https://play.google.com/store/apps/details?id=" + packageName
    }

    override fun onCleared() {
        super.onCleared()
        compositeDisposable.clear()
    }
}